{==================================================================
  QddgDsnSpn Unit

  This unit implements the TddgDesignSpinner component, which
  descends from TddgSpinner and displays a custom mouse cursor when
  the mouse is positioned over one of the buttons, and allows the
  user to change the value of the spinner at design-time using the
  mouse.

  Copyright  2001 by Ray Konopka
==================================================================}

unit QddgDsnSpn;

interface

uses
  SysUtils, Classes, Qt, QddgSpin;
  (*
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ddgSpin;
  *)

type
  TddgDesignSpinner = class( TddgSpinner )
  private
    // VCL->CLX:  Custom cursor stored in QCursorH field
    FThumbCursor: QCursorH;

    (*
    // VCL->CLX:  Custom cursors and design-time interactions are
    //            handled differently under CLX. The following
    //            block is VCL-specific.

    FThumbCursor: HCursor;

    // Window Message Handling Method
    procedure WMSetCursor( var Msg : TWMSetCursor );
      message wm_SetCursor;

    // Component Message Handling Method
    procedure CMDesignHitTest( var Msg: TCMDesignHitTest );
      message cm_DesignHitTest;
    *)
  protected
    procedure Change; override;

    // VCL->CLX:  The following two methods are overridden for CLX
    procedure MouseMove( Shift: TShiftState;
                         X, Y: Integer ); override;
    function DesignEventQuery( Sender: QObjectH;
                               Event: QEventH ): Boolean; override;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  end;

implementation

(*
// VCL->CLX:  CLX does not support cursor resources
{$R DdgDsnSpn.res}                // Link in custom cursor resource
*)

uses
  Types, QControls, QForms;             // VCL->CLX:  Add CLX units

// VCL->CLX:  Two arrays of bytes (one for the image and one for
//            the mask) are used to represent custom cursors in CLX

const
  Bits: array[0..32*4-1] of Byte = (
    $00, $30, $00, $00, $00, $48, $00, $00,
    $00, $48, $00, $00, $00, $48, $00, $00,
    $00, $48, $00, $00, $00, $4E, $00, $00,
    $00, $49, $C0, $00, $00, $49, $30, $00,
    $00, $49, $28, $00, $03, $49, $24, $00,
    $04, $C0, $24, $00, $04, $40, $04, $00,
    $02, $40, $04, $00, $02, $00, $04, $00,
    $01, $00, $04, $00, $01, $00, $04, $00,
    $00, $80, $08, $00, $00, $40, $08, $00,
    $00, $40, $08, $00, $00, $20, $10, $00,
    $00, $20, $10, $00, $00, $7F, $F8, $00,
    $00, $7F, $F8, $00, $00, $7F, $E8, $00,
    $00, $7F, $F8, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00 );

  Mask: array[0..32*4-1] of Byte = (
    $00, $30, $00, $00, $00, $78, $00, $00,
    $00, $78, $00, $00, $00, $78, $00, $00,
    $00, $78, $00, $00, $00, $7E, $00, $00,
    $00, $7F, $C0, $00, $00, $7F, $F0, $00,
    $00, $7F, $F8, $00, $03, $7F, $FC, $00,
    $07, $FF, $FC, $00, $07, $FF, $FC, $00,
    $03, $FF, $FC, $00, $03, $FF, $FC, $00,
    $01, $FF, $FC, $00, $01, $FF, $FC, $00,
    $00, $FF, $F8, $00, $00, $7F, $F8, $00,
    $00, $7F, $F8, $00, $00, $3F, $F0, $00,
    $00, $3F, $F0, $00, $00, $7F, $F8, $00,
    $00, $7F, $F8, $00, $00, $7F, $E8, $00,
    $00, $7F, $F8, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00 );


{===============================}
{== TddgDesignSpinner Methods ==}
{===============================}

constructor TddgDesignSpinner.Create( AOwner: TComponent );
var
  BitsBitmap: QBitmapH;
  MaskBitmap: QBitmapH;
begin
  inherited Create( AOwner );

  (*
  // VCL->CLX:  No LoadCursor in CLX
  FThumbCursor := LoadCursor( HInstance, 'DdgDSNSPN_BTNCURSOR' );
  *)

  // VCL->CLX:  Byte arrays are used to create a custom cursor
  BitsBitmap := QBitmap_create( 32, 32, @Bits, False );
  MaskBitmap := QBitmap_create( 32, 32, @Mask, False );
  try
    FThumbCursor := QCursor_create( BitsBitmap, MaskBitmap, 8, 0 );
  finally
    QBitmap_destroy( BitsBitmap );
    QBitmap_destroy( MaskBitmap );
  end;
end;


destructor TddgDesignSpinner.Destroy;
begin
  (*
  VCL->CLX:  In CLX, use QCursor_Destroy instead of DestroyCursor
  DestroyCursor( FThumbCursor );       // Release GDI cursor object
  *)
  QCursor_Destroy( FThumbCursor );
  inherited Destroy;
end;


// If the mouse is over one of the buttons, then change cursor to
// the custom cursor that resides in the DdgDsnSpn.res
// resource file

(*
// VCL->CLX:  There is no wm_SetCursor in CLX
procedure TddgDesignSpinner.WMSetCursor( var Msg: TWMSetCursor );
begin
  if MouseOverButton( btMinus ) or MouseOverButton( btPlus ) then
    SetCursor( FThumbCursor )
  else
    inherited;
end;
*)

// VCL->CLX:  Override MouseMove to handle displaying custom cursor

procedure TddgDesignSpinner.MouseMove( Shift: TShiftState;
                                      X, Y: Integer );
begin
  if MouseOverButton( btMinus ) or MouseOverButton( btPlus ) then
    QWidget_setCursor( Handle, FThumbCursor )
  else
    QWidget_UnsetCursor( Handle );
  inherited;
end;


(*
// VCL->CLX:  cm_DesignHitTest does not exist in CLX.  Instead,
//            override the DesignEventQuery method (see below).

procedure TddgDesignSpinner.CMDesignHitTest( var Msg:
                                            TCMDesignHitTest );
begin
  // Handling this component message allows the Value of the
  // spinner to be changed at design-time using the left mouse
  // button.  If the mouse is positioned over one of the buttons,
  // then set the Msg.Result value to 1. This instructs Delphi to
  // allow mouse events to "get through to" the component.

  if MouseOverButton( btMinus ) or MouseOverButton( btPlus ) then
    Msg.Result := 1
  else
    Msg.Result := 0;
end;
*)


function TddgDesignSpinner.DesignEventQuery( Sender: QObjectH;
                                         Event: QEventH ): Boolean;
var
  MousePos: TPoint;
begin
  Result := False;
  if ( Sender = Handle ) and
     ( QEvent_type(Event) in [QEventType_MouseButtonPress,
                              QEventType_MouseButtonRelease,
                              QEventType_MouseButtonDblClick]) then
  begin
    // Note: extracting MousePos is not actually needed in this
    //       example, but if you need to get the position of the
    //       mouse, this is how you do it.

    MousePos := Point( QMouseEvent_x( QMouseEventH( Event ) ),
                       QMouseEvent_y( QMouseEventH( Event ) ) );

    if MouseOverButton( btMinus ) or MouseOverButton( btPlus ) then
      Result := True
    else
      Result := False;
  end;
end;


procedure TddgDesignSpinner.Change;
var
  Form: TCustomForm;
begin
  inherited Change;

  // Force the Object Inspector to update what it shows for the
  // Value property of the spinner when changed via the mouse.

  if csDesigning in ComponentState then
  begin
    Form := GetParentForm( Self );

    (*
    // VCL->CLX:  Form.Designer replaced with DesignerHook in CLX
    if ( Form <> nil ) and ( Form.Designer <> nil ) then
      Form.Designer.Modified;
    *)

    if ( Form <> nil ) and ( Form.DesignerHook <> nil ) then
      Form.DesignerHook.Modified;
  end;
end;
 
end.
