unit Bank_s;

interface

uses
  CORBA, Bank_i, Bank_c;

type
  TAccountSkeleton = class;

  TAccountSkeleton = class(CORBA.TCorbaObject, Bank_i.Account)
  private
    FImplementation : Account;
  public
    constructor Create(const InstanceName: string; const Impl: Account);
    destructor Destroy; override;
    function GetImplementation : Account;
    procedure deposit ( const amount : Single);
    procedure withdraw ( const amount : Single);
    function  balance : Single;
  published
    procedure _deposit(const _Input: CORBA.InputStream; _Cookie: Pointer);
    procedure _withdraw(const _Input: CORBA.InputStream; _Cookie: Pointer);
    procedure _balance(const _Input: CORBA.InputStream; _Cookie: Pointer);
  end;


implementation

constructor TAccountSkeleton.Create(const InstanceName : string; const Impl : Bank_i.Account);
begin
  inherited;
  inherited CreateSkeleton(InstanceName, 'Account', 'IDL:Bank/Account:1.0');
  FImplementation := Impl;
end;

destructor TAccountSkeleton.Destroy;
begin
  FImplementation := nil;
  inherited;
end;

function TAccountSkeleton.GetImplementation : Bank_i.Account;
begin
  result := FImplementation as Bank_i.Account;
end;

procedure TAccountSkeleton.deposit ( const amount : Single);
begin
  FImplementation.deposit( amount);
end;

procedure TAccountSkeleton.withdraw ( const amount : Single);
begin
  FImplementation.withdraw( amount);
end;

function  TAccountSkeleton.balance : Single;
begin
  Result := FImplementation.balance;
end;

procedure TAccountSkeleton._deposit(const _Input: CORBA.InputStream; _Cookie: Pointer);
var
  _Output : CORBA.OutputStream;
  _amount : Single;
begin
  _Input.ReadFloat(_amount);
  deposit( _amount);
  GetReplyBuffer(_Cookie, _Output);
end;

procedure TAccountSkeleton._withdraw(const _Input: CORBA.InputStream; _Cookie: Pointer);
var
  _Output : CORBA.OutputStream;
  _amount : Single;
begin
  _Input.ReadFloat(_amount);
  try
    withdraw( _amount);
  except on E: UserException do
    begin
      GetExceptionReplyBuffer(_Cookie, _Output);
      E.WriteExceptionInfo(_Output);
      exit
    end;
  end;
  GetReplyBuffer(_Cookie, _Output);
end;

procedure TAccountSkeleton._balance(const _Input: CORBA.InputStream; _Cookie: Pointer);
var
  _Output : CORBA.OutputStream;
  _Result : Single;
begin
  _Result := balance;
  GetReplyBuffer(_Cookie, _Output);
  _Output.WriteFloat(_Result);
end;


initialization


end.