{
NOTE:
-----

This Web Service is implemented here as a WAD executable just
for ease... the correct implementation in a production environment
should be an ISAPI/NSAPI dll
}

{ Invokable implementation declaration unit for TAuthenticator,
  which implements IAuthenticator }

unit AuthenticatorImpl;

interface

uses
  AuthenticatorIntf, InvokeRegistry;

type
  TAuthenticator = class(TInvokableClass, IAuthenticator)
  private
    function GetStringGuid : String;
    function LocateSession (const SessionHandle : String): boolean;
  public
    function IsValidSession(const SessionHandle: String): Boolean;
      stdcall;
    function Login(const User: String; const Password: String;
      var SessionHandle: String): Boolean; stdcall;
    function Logout(const SessionHandle: String): Boolean; stdcall;
    function AddSessionData(const SessionHandle: String;
      const Data: String): Boolean; stdcall;
    function GetSessionData(const SessionHandle: String): String; stdcall;
  end;

implementation
uses
  SyncObjs, DB,SysUtils, ActiveX, UwmAuthServer, UdtmSessions;

var
  CS : TCriticalSection;

{ TAuthenticator }

function TAuthenticator.AddSessionData(const SessionHandle: String;
  const Data: String): Boolean;
begin
  //checking if session is already established
  Result := True;
  if not LocateSession(SessionHandle) then
  begin
    Result :=False;
    Exit;
  end;
  //ok, let's add the data
  CS.Enter;
  try
    dtmSessions.cdsSessions.Edit;
    dtmSessions.cdsSessions.FieldByName('SessionData').AsString := Data;
    dtmSessions.cdsSessions.Post;
    dtmSessions.cdsSessions.SaveToFile;
  finally
    CS.Leave;
  end;
end;

function TAuthenticator.GetSessionData(
  const SessionHandle: String): String;
begin
  Result := '';
  if not LocateSession(SessionHandle) then
    Exit;
  Result := dtmSessions.cdsSessions.FieldByName ('SessionData').AsString;
end;

function TAuthenticator.GetStringGuid: String;
var
  GUID : TGUID;
begin
  CoCreateGuid(GUID);
  Result := GUIDToString(GUID);
end;

function TAuthenticator.IsValidSession(
  const SessionHandle: String): Boolean;
begin
  CS.Enter;
  try
    Result := dtmSessions.cdsSessions.Locate('SessionHandle',SessionHandle,[]);
  finally
    CS.Leave;
  end;
end;

function TAuthenticator.LocateSession (const SessionHandle : String): boolean;
begin
  //Locating the session
  Result := IsValidSession(SessionHandle);
end;

function TAuthenticator.Login(const User, Password: String;
  var SessionHandle: String): Boolean;
begin
  //Here you would check for the validity of the User/Password pair
  //The implementation below is just a simple substitute of a better
  //mechanism.
  Result := True;
  if (CompareText (User,'daniel')<>0) or (password <> 'test') then
  begin
    Result := False;
    Exit;
  end;
  //let's see if the user is already logged in
  CS.Enter;
  try
    if dtmSessions.cdsSessions.Locate('SessionUser',User,[loCaseInsensitive]) then
      raise Exception.Create('User already logged in');
    //from here on, we will assume that the login is ok, let's create a session!
    SessionHandle := GetStringGuid;
    dtmSessions.cdsSessions.Insert;
    dtmSessions.cdsSessions.FieldByName('SessionHandle').AsString := SessionHandle;
    dtmSessions.cdsSessions.FieldByName('SessionUser').AsString := User;
    dtmSessions.cdsSessions.Post;
    dtmSessions.cdsSessions.SaveToFile;
  finally
    CS.Leave;
  end;
end;

function TAuthenticator.Logout(const SessionHandle: String): Boolean;
begin
  //Locating the session
  if not LocateSession(SessionHandle) then
  begin
    Result := False;
    Exit;
  end;
  //deleting the session
  CS.Enter;
  try
    dtmSessions.cdsSessions.Delete;
    dtmSessions.cdsSessions.SaveToFile;
  finally
    CS.Leave;
  end;
  Result := True;
end;

initialization
  InvRegistry.RegisterInvokableClass(TAuthenticator);
  CS := TCriticalSection.Create;
finalization
  CS.Free;
end.
