unit ddg6WebSnapComps;

interface

uses Classes, WebComp, SysUtils, MidComp;

type

  Tddg6BaseWebSnapComponent = class(TWebContainedComponent, IWebContent)
  protected
    { IWebContent }
    function Content(Options: TWebContentOptions; ParentLayout: TLayout): string;
    function GetHTML: string; virtual; abstract;
  end;

type


  Tddg6WebFile = class(Tddg6BaseWebSnapComponent)
  private
    FHTMLFile: TFilename;
    FHTMLDoc: TStrings;
    FFileInDFM: Boolean;
    procedure SetHTMLDoc(const Value: TStrings);
    procedure SetHTMLFile(const Value: TFilename);
    procedure SetFileInDFM(const Value: Boolean);

  protected
    function GetHTML: string; override;
    function LoadFile: string;
  public
    constructor Create(AComponent: TComponent); override;
    destructor Destroy; override;
  published
    property FileInDFM: Boolean read FFileInDFM write SetFileInDFM;
    property HTMLDoc: TStrings read FHTMLDoc write SetHTMLDoc;
    property HTMLFile: TFilename read FHTMLFile write SetHTMLFile;
  end;

  Tddg6HTMLCode = class(Tddg6BaseWebSnapComponent)
  private
    FHTML: TStrings;
    procedure SetHTML(const Value: TStrings);
  protected
    function GetHTML: string; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property HTML: TStrings read FHTML write SetHTML;
  end;


  procedure Register;


implementation

uses WebForm, MidItems, NixUtils;

type
  THelper = class(TWebComponentsEditorHelper)
  protected
     function ImplCanAddClassHelper(AEditor: TComponent; AParent: TComponent; AClass: TClass): Boolean; override;
  end;

var
 ddg6Helper: THelper;

procedure Register;
begin
  RegisterWebComponents([Tddg6WebFile, Tddg6HTMLCode], ddg6Helper);
end;

{ THelper }
function THelper.ImplCanAddClassHelper(AEditor: TComponent; AParent: TComponent; AClass: TClass): Boolean;
begin
  // Return True to indicate that AParent is a valid parent of Tddg6WebLink.
  Result := AEditor.InheritsFrom(TCustomLayoutGroup) or
    AEditor.InheritsFrom(TCustomAdapterForm);
end;

{ Tddg6BaseWebSnapComponent }
function Tddg6BaseWebSnapComponent.Content(Options: TWebContentOptions;
  ParentLayout: TLayout): string;
var
  Intf: ILayoutWebContent;
begin
  if Supports(ParentLayout, ILayoutWebContent, Intf) then
    Result := Intf.LayoutField(GetHTML, nil)
  else
    Result := GetHTML;
end;

{ Tddg6WebFile }

constructor Tddg6WebFile.Create(AComponent: TComponent);
begin
  inherited;
  FHTMLDoc := TStringList.Create;
  FileInDFM := False;
end;

destructor Tddg6WebFile.Destroy;
begin
  inherited;
  FHTMLDoc.Free;
end;

function Tddg6WebFile.GetHTML: string;
begin
  if FileInDFM then
  begin
    Result := HTMLDoc.Text;
  end else
  begin
   Result := LoadFile;
  end;
end;

function Tddg6WebFile.LoadFile: string;
var
  FS: TFileStream;
  SS: TStringStream;
begin
  if (HTMLFile <> '') and FileExists(HTMLFile) then
  begin
    FS := TFileStream.Create(HTMLFile, fmOpenRead);
    SS := TStringStream.Create('');
    try
      SS.CopyFrom(FS, FS.Size);
      Result := SS.DataString;
    finally
      FS.Free;
      SS.Free;
    end;
  end else
  begin
    Result := '';  // if the file doesn't exist, then return nothing
  end;
end;

procedure Tddg6WebFile.SetFileInDFM(const Value: Boolean);
begin
  // Note: if this is true, then the contents of the file are added to the HTMLDoc property, and thus
  // written to the DFM file.  If that is the case, and you make changes to the file, they _won't_ be reflected
  // in the component.  If this is false, then the component will hunt up the file each time that it is asked for
  // and sent as Content
  if FFileInDFM <> Value then
  begin
    FFileInDFM := Value;
    HTMLDoc.Clear;
    if FileInDFM then
    begin
      HTMLDoc.Add(LoadFile);
    end;
  end;
end;

procedure Tddg6WebFile.SetHTMLDoc(const Value: TStrings);
begin
  FHTMLDoc := Value;
end;

procedure Tddg6WebFile.SetHTMLFile(const Value: TFilename);
begin
  FHTMLFile := Value;
  if FileInDFM then
  begin
    HTMLDoc.Clear;
    HTMLDoc.Add(LoadFile);
  end;
end;


{ Tddg6HTMLCode }

constructor Tddg6HTMLCode.Create(AOwner: TComponent);
begin
  inherited;
  FHTML := TStringList.Create;
end;

destructor Tddg6HTMLCode.Destroy;
begin
  FHTML.Free;
  inherited;
end;

function Tddg6HTMLCode.GetHTML: string;
begin
  Result := FHTML.Text;
end;

procedure Tddg6HTMLCode.SetHTML(const Value: TStrings);
begin
  FHTML.Assign(Value);
end;

initialization
  // Helper is used at design-time to indicate which components can parent
  // Tddg6WebLink.
  ddg6Helper := THelper.Create;
finalization
  UnregisterWebComponents([Tddg6HTMLCode, Tddg6WebFile]);
  ddg6Helper.Free;
end.
