
unit wmHome;

interface

uses
  Windows, Messages, SysUtils, Classes, HTTPApp, WebModu, HTTPProd, ReqMulti,
  WebUsers, WebSess, WebDisp, WebAdapt, WebComp;

type
  THome = class(TWebAppPageModule)
    PageProducer: TPageProducer;
    WebAppComponents: TWebAppComponents;
    ApplicationAdapter: TApplicationAdapter;
    EndUserSessionAdapter: TEndUserSessionAdapter;
    PageDispatcher: TPageDispatcher;
    AdapterDispatcher: TAdapterDispatcher;
    WebDispatcher: TWebDispatcher;
    LocateFileService: TLocateFileService;
    SessionsService: TSessionsService;
    WebUserList: TWebUserList;
    ActionLoginForm: TEndUserAdapterLoginFormAction;
    ActionLogout: TEndUserAdapterLogoutAction;
    AdaptDisplayName: TAdapterEndUserDisplayNameField;
    AdaptLoggedIn: TAdapterEndUserLoggedInField;
    procedure SessionsServiceEndSession(ASender: TObject;
      ASession: TAbstractWebSession; AReason: TEndSessionReason);
    procedure LocateFileServiceFindStream(ASender: TObject;
      AComponent: TComponent; const AFileName: String;
      var AFoundStream: TStream; var AOwned, AHandled: Boolean);
  private
    FCurrentUserName: string;
    { Private declarations }
  public
    { Public declarations }
    property CurrentUserName: string write FCurrentUserName;
  end;

  function Home: THome;

  function IniFileName: string;

  var
    Lock: TMultiReadExclusiveWriteSynchronizer;

  const
    sFavoriteMovie = 'FavoriteMovie';
    sPasswordHint = 'PasswordHint';
    sLikesChocolate = 'LikesChocolate';
    sIniFileName = 'DDG6Demo.ini';


implementation

{$R *.dfm}  {*.html}

uses WebReq, WebCntxt, WebFact, Variants, IniFiles;

function Home: THome;
begin
  Result := THome(WebContext.FindModuleClass(THome));
end;

function IniFileName: string;
begin
  Result := ExtractFilePath(GetModuleName(HInstance)) + sIniFileName;
end;

procedure THome.SessionsServiceEndSession(ASender: TObject;
  ASession: TAbstractWebSession; AReason: TEndSessionReason);
var
  IniFile: TIniFile;
begin
  //Save out the preferences here
  Lock.BeginWrite;
  if FCurrentUserName <> '' then
  begin
    try
      IniFile := TIniFile.Create(IniFileName);
      try
        IniFile.WriteString(FCurrentUserName, sFavoriteMovie, ASession.Values[sFavoriteMovie]);
        IniFile.WriteString(FCurrentUserName, sPassWordHint, ASession.Values[sPasswordHint]);
        IniFile.WriteString(FCurrentUserName, sLikesChocolate, ASession.Values[sLikesChocolate]);
      finally
        IniFile.Free;
      end;
    finally
      Lock.EndWrite
    end;
  end;
end;

procedure THome.LocateFileServiceFindStream(ASender: TObject;
  AComponent: TComponent; const AFileName: String;
  var AFoundStream: TStream; var AOwned, AHandled: Boolean);
begin
  if Pos('WMEMBEDDED', UpperCase(AFileName)) > 0 then // we are hunting up the Embedded file
  begin
    AFoundStream := TResourceStream.Create(hInstance, 'EMBEDDEDHTML', 'HTML');
    AHandled := True; // no need to look further
  end;
end;

initialization
  if WebRequestHandler <> nil then
    WebRequestHandler.AddWebModuleFactory(TWebAppPageModuleFactory.Create(THome, TWebPageInfo.Create([wpPublished {, wpLoginRequired}], '.html'), caCache));
  Lock := TMultiReadExclusiveWriteSynchronizer.Create;
finalization
  Lock.Free;
end.
